--[[
The visual button for an instance of an action.  Based on settings, the button 
can look different.  It can look like a menu item, with the icon and spell name 
showing side by side, or look like a typical action button with just the icon 
showing.  Other appearances are possible as well.  This class controls 
implementing the standard appearance of the button, and how the visual effects 
actually appear (e.g., just the icon blinks, but the text does not, or the icon 
is highlighted, but the text is not).

An ActionButton instance is allocated for each action for a UnitGroupHeader.
Each ActionButton instance is associated with a single UnitGroupHeader and a 
single Action.

When an ActionButton is shown, it adjusts it's scale to the currently set scale,
then it tells the associated Action to identify the visual effects based on the 
state of the action and the unit the ActionButton is associated with.
--]]
uab.classes.ActionButton = uab.mixins.aceOO.Class()

function uab.classes.ActionButton.TimerEvent(self, event, elapsed)
    self[event](self, elapsed)
end

function uab.classes.ActionButton.prototype:init(header, action)
	uab.classes.ActionButton.super.prototype.init(self)
	
	self.header = header
    self.savedattrs = {}

    local framename = "uab_"..self.header.name.."_ActionBtn"..action.number

    self.timername = "ActionButtonTimer_"..framename

    uab:RegisterMetro(
        self.timername,
        uab.classes.ActionButton.TimerEvent,
        0,
        self,
        "TimerEvent")
	
	--[[
	Create the action button frame
	--]]
    self.frame = CreateFrame(
        "Button",
        framename,
        nil,
        "UAB_StandardActionButtonTemplate")
    
    self.frame.uabobj = self
    self.frame:Show()

	local count = self.frame:CreateFontString(framename.."ItemCount", "ARTWORK", "NumberFontNormal")
	self.frame.itemcount = count
    
	count:SetText("")
	count:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMRIGHT", -2, 2)
    local fontName, fontHeight, fontFlags = count:GetFont()
    count:SetFont(fontName, 10, fontFlags)

	local cooldown = self.frame:CreateFontString(framename.."Cooldown", "ARTWORK", "NumberFontNormal")
    self.frame.cooldown = cooldown
    
	cooldown:SetText("")
	cooldown:SetPoint("TOP", self.frame, "TOP", 0, -2)
    fontName, fontHeight, fontFlags = cooldown:GetFont()
    cooldown:SetFont(fontName, 10, fontFlags)

	local border = self.frame:CreateTexture(framename.."Border", "OVERLAY")
    self.frame.border = border
    border:SetTexture("Interface\\AddOns\\UnitActionBars\\images\\swborder")
    border:SetBlendMode("ADD")
    border:SetWidth(40)
    border:SetHeight(40)
    border:ClearAllPoints()
    border:SetPoint("CENTER", self.frame, "CENTER", 0, -1)
    border:Hide()

    self.frame:SetAttribute("useparent-unit", true)
    self.frame:SetAttribute("useparent-unitsuffix", true)

    self:SetAction(action)

    self.frame:SetScript("OnShow", function(self) self.uabobj:OnShow() end)
    self.frame:SetScript("OnHide", function(self) self.uabobj:OnHide() end)
    self.frame:SetScript("OnEnter", function(self, motion) self.uabobj:ShowTooltip() end)
    self.frame:SetScript("OnLeave", function(self, motion) GameTooltip:Hide() end)

    self:Enable()
	self:SetParent()
    self.frame:Hide()
end

function uab.classes.ActionButton.prototype:SetParent()
    if self.header.frame then
        self.frame:SetParent(self.header.frame)
    	self.header.frame:SetAttribute("addchild", self.frame)
    end
end

function uab.classes.ActionButton.prototype:SetScale(scale)
    self.frame:SetScale(scale)
end

function uab.classes.ActionButton.prototype:SetAction(action)
    if self.action then
        self.action:RemoveButton(self)
    end
    self.action = action
    if self.action then
        self.action:AddButton(self)
    end
end

function uab.classes.ActionButton.prototype:ClearAttributes(attributes)
	for name, _ in pairs(attributes) do
		if self.savedattrs[name] then
			self.frame:SetAttribute(name, self.savedattrs[name])
		else
			self.frame:SetAttribute(name, nil)
		end
	end
end

function uab.classes.ActionButton.prototype:SetAttributes(attributes)
	for name, val in pairs(attributes) do
		self.savedattrs[name] = self.frame:GetAttribute(name)
		self.frame:SetAttribute(name, val)
	end
end

function uab.classes.ActionButton.prototype:Show()
	self.action:ShowButton(self)
end

function uab.classes.ActionButton.prototype:Hide()
	self.frame:Hide()
end

function uab.classes.ActionButton.prototype:Enable()
    self.frame:SetAttribute("showstates", self.header.states.min.."-"..self.header.states.max)
    self.frame:SetAttribute("hidestates", nil)
	self.enabled = true
    
    self:StayOpenChanged()
end

function uab.classes.ActionButton.prototype:Disable()
    self.frame:SetAttribute("showstates", nil)
    self.frame:SetAttribute("hidestates", "*")
    self.enabled = nil

    self:StayOpenChanged()    
end

function uab.classes.ActionButton.prototype:StayOpenChanged()
    if not self.header.group:IsStayOpen() or not self.enabled then
        self.frame:SetAttribute("newstate", "*:0")
    else
        self.frame:SetAttribute("newstate", nil)
    end
end

function uab.classes.ActionButton.prototype:Frame()
	return self.frame
end

function uab.classes.ActionButton.prototype:BorderColor(r, g, b, a)
    if a == 0 then
        self.frame.border:Hide()
    else
        self.frame.border:SetVertexColor(r,g,b,a)
        self.frame.border:Show()
    end
end

function uab.classes.ActionButton.prototype:SetAlpha(alpha)
    self.frame:SetAlpha(alpha)
end

function uab.classes.ActionButton.prototype:StartFlash()
    if not self.flashing then
        self.flashing = { timer = 0.5, direction = 1 }
    end
end

function uab.classes.ActionButton.prototype:EndFlash()
	if self.flashing then
        self.flashing = nil
    end
end

function uab.classes.ActionButton.prototype:StatusChanged()
    if self.action then
        self.action:ShowButton(self)
    end
end

function uab.classes.ActionButton.prototype:TimerEvent(elapsed)
    if self.flashing then
        self.flashing.timer = self.flashing.timer - elapsed
        if self.flashing.timer < 0 then
            self.flashing.timer = .5
            if self.flashing.direction then
                self.flashing.direction = nil
            else
                self.flashing.direction = 1
            end
        else
            if self.flashing.direction then
                local a = self.flashing.timer * 2
                self.frame:SetAlpha(a)
            else
                local a = 1 - self.flashing.timer * 2
                self.frame:SetAlpha(a)
            end
        end
    end

    local cooldown
    
    if self.action then    
        cooldown = self.action:GetData():GetCooldown()
    end

    if cooldown then
    	self.cooldown = cooldown
        self.frame.cooldown:SetText(cooldown)
        self.frame.cooldown:Show()
    elseif self.cooldown then
    	self.cooldown = nil
        self.frame.cooldown:SetText("")
        self.frame.cooldown:Hide()
    end
    
    self:StatusChanged()
end

function uab.classes.ActionButton.prototype:IsUnit(unit)
	return unit == SecureButton_GetModifiedUnit(self.frame)
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.ActionButton.prototype:ShowTooltip()
	self.action:ShowTooltip(self.frame)
end

function uab.classes.ActionButton.prototype:OnShow()
    self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
    uab:StartMetro(self.timername)
--    uab.modules.UnitStatus:NotifyButtonOpen(self)
end

function uab.classes.ActionButton.prototype:OnHide()
    uab:StopMetro(self.timername)
--    uab.modules.UnitStatus:NotifyButtonClosed(self)
end
